<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Tom Needham <tom@owncloud.com>
 * @author Frank Karlitscheck <frank@owncloud.com>
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright (C) 2016 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit\Handlers;

use OC\Files\Filesystem;
use OC\Files\View;
use OCP\Files\FileInfo;
use OCP\IUser;
use Punic\Exception;

class Files extends Base {
	static $beforeDeleteFileInfo = [];

	static public function excludeLog($path) {
		$uid = "";
		$path = ltrim($path, "/");
		/**
		 * Making assumption that path has to be $uid/files/
		 * So if the path when exploded doesn't make up to this
		 * pattern, then this method will return true. Else return
		 * false.
		 */
		$splitPath = explode("/", $path);
		if (count($splitPath) < 2) {
			//For example if the path has /avatars
			return true;
		} else {
			//For example if the path has /avatars/12
			if ($splitPath[1] !==  'files') {
				return true;
			} else {
				//After doing ltrim chances are bit low for empty. Still who knows
				if ($splitPath[0] === "") {
					return true;
				}
				$uid = $splitPath[0];
			}
		}
		$userPathStarts = $uid . "/files/";

		/**
		 * Final check if the path has $uid/files/ in it
		 * if so then get the data logged. Else exclude
		 * them from the logger
		 */
		if (strpos($path, $userPathStarts) !== false) {
			return false;
		}

		return true;
	}

	// Normal files actions
	static public function rename($params) {
		$result = self::excludeLog($params['newpath']);
		if ($result === true) {
			return;
		}
		$logger = self::getLogger();
		$fileInfo = \OC::$server->getRootFolder()->get($params['newpath']);

		$fileId = $fileInfo->getId();
		if ($fileId === null) {
			return;
		}

		self::getLogger()->log(
			'Rename "{oldpath}"{tags} with fileid: {oldid} to "{newpath}" with fileid: {newid} by {actor}, owner: {owner}',
			[
				'owner' => $fileInfo->getOwner()->getUID(),
				'oldpath' => $params['oldpath'],
				'newpath' => $params['newpath'],
				'tags' => $logger->getTagStringForFile($fileId),
				'newid' => $fileInfo->getId(),
				'oldid' => $fileInfo->getId(),
			]
		);
	}
	static public function copy($params) {
		$result = self::excludeLog($params['oldpath']);
		if ($result === true) {
			return;
		}

		$logger = self::getLogger();
		$oldFileInfo = \OC::$server->getRootFolder()->get($params['oldpath']);
		$newFileInfo = \OC::$server->getRootFolder()->get($params['newpath']);

		self::getLogger()->log(
			'Copy "{oldpath}"{tags} with fileid: {oldid} to "{newpath}" with fileid {newid} by {actor}, owner: {owner}',
			[
				'owner' => $oldFileInfo->getOwner()->getUID(),
				'oldpath' => $params['oldpath'],
				'newpath' => $params['newpath'],
				'tags' => $logger->getTagStringForFile($oldFileInfo->getId()),
				'oldid' => $oldFileInfo->getId(),
				'newid' => $newFileInfo->getId(),
			]
		);
	}
	static public function create($params) {
		$result = self::excludeLog($params['path']);
		if ($result === true) {
			return;
		}
		$fileInfo = \OC::$server->getRootFolder()->get($params['path']);
		self::getLogger()->log(
			'Create "{path}"{tags} with fileid: {id} by {actor}, owner: {owner}',
			[
				'owner' => $fileInfo->getOwner()->getUID(),
				'path' => $fileInfo->getPath(),
				'tags' => '',// Since we only show the tags of this file, this makes no sense yet.
				'id' => $fileInfo->getId(),
			]
		);
	}
	static public function update($params) {
		$fileInfo = \OC::$server->getRootFolder()->get($params['path']);
		$result = self::excludeLog($fileInfo->getPath());
		if ($result === true) {
			return;
		}
		$logger = self::getLogger();

		$logger->log(
			'Update "{path}"{tags} with fileid: {id} by {actor}, owner: {owner}',
			[
				'owner' => $fileInfo->getOwner()->getUID(),
				'path' => $params['path'],
				'tags' => $logger->getTagStringForFile($fileInfo->getId()),
				'id' => $fileInfo->getId(),
			]
		);
	}
	static public function read($params) {
		$result = self::excludeLog(self::getAbsolutePath($params[Filesystem::signal_param_path]));
		if ($result === true) {
			return;
		}

		$logger = self::getLogger();
		$fileId = self::getFileId($params[Filesystem::signal_param_path]);
		if ($fileId === null) {
			return;
		}

		$logger->log(
			'Read "{path}"{tags} with fileid: {id} by {actor}, owner: {owner}',
			[
				'owner' => self::getOwner($params[Filesystem::signal_param_path]),
				'path' => self::getAbsolutePath($params[Filesystem::signal_param_path]),
				'tags' => $logger->getTagStringForFile($fileId),
				'id' => $fileId,
			]
		);
	}

	static public function beforeDelete($params) {
		//This function helps to preserve any delete operations pending
		try {
			$fileInfo = \OC::$server->getRootFolder()->get($params['path']);
			$result = self::excludeLog($fileInfo->getPath());
			if ($result === true) {
				return;
			}
		} catch (\Exception $e) {
			/*
			 * There are cases where the thumbnails folders when tried to delete
			 * they don't exist and it causes lot of noise in the log file with
			 * exception. This is just to prevent that noise.
			 */

			return;
		}

		array_push(self::$beforeDeleteFileInfo, $fileInfo);
	}

	static public function delete($params) {
		$logger = self::getLogger();
		//This is to make sure that delete is successfully done.
		while ($fileInfo = array_shift(self::$beforeDeleteFileInfo)) {
			$logger->log(
				'Delete "{path}"{tags} with fileid: {id} by {actor}, owner: {owner}',
				[
					'owner' => $fileInfo->getOwner()->getUID(),
					'path' => $fileInfo->getPath(),
					'tags' => $logger->getTagStringForFile($fileInfo->getId()),
					'id' => $fileInfo->getId(),
				]
			);
		}
	}

	// Trashbin
	static public function trash_post_restore($params) {
		$logger = self::getLogger();
		$user = $logger->getSessionUser();
		if ($user instanceof IUser) {
			$uid = $user->getUID();
		} else {
			$view = Filesystem::getView();
			$uid = $view->getOwner('');
		}
		$view = new \OC\Files\View('/' . $uid);
		$fileId = self::getFileId($params['filePath']);
		if ($fileId === null) {
			return;
		}

		$logger->log(
			'Restore "{oldpath}"{tags} to "{newpath}" with fileid: {id} by {actor}, owner: {owner}',
			[
				'owner' => $view->getOwner('files/' . $params['filePath']),
				'oldpath' => $view->getAbsolutePath('files_trashbin/files/' . $params['trashPath']),
				'newpath' => self::getAbsolutePath($params['filePath']),
				'tags' => $logger->getTagStringForFile($fileId),
				'id' => $fileId
			]
		);
	}
	static public function trash_delete($params) {
		$logger = self::getLogger();
		$user = $logger->getSessionUser();
		if ($user instanceof IUser) {
			$uid = $user->getUID();
		} else {
			$view = Filesystem::getView();
			$uid = $view->getOwner('');
		}
		$view = new \OC\Files\View('/' . $uid);
		$fileId = $view->getFileInfo($params['path'])->getId();
		if ($fileId === null) {
			return;
		}

		$logger->log(
			'DeleteFromTrash "{path}"{tags} by {actor}, owner: {owner}',
			[
				'owner' => $view->getOwner($params['path']),
				'path' => $view->getAbsolutePath($params['path']),
				'tags' => $logger->getTagStringForFile($fileId),
			]
		);
	}

	static protected function getAbsolutePath($path) {
		return Filesystem::getView()->getAbsolutePath($path);
	}

	static protected function getFileId($path) {
		if (Filesystem::getView() instanceof View) {
			if (Filesystem::getView()->getFileInfo($path) instanceof FileInfo) {
				return Filesystem::getView()->getFileInfo($path)->getId();
			}
		}
		return null;
	}

	static protected function getOwner($path) {
		return Filesystem::getOwner($path);
	}

	// Versions
	static public function version_delete($params) {
		$logger = self::getLogger();

		switch($params['trigger']) {
			case \OCA\Files_Versions\Storage::DELETE_TRIGGER_MASTER_REMOVED:
				$triggerMsg = 'deleted master file';
				break;
			case \OCA\Files_Versions\Storage::DELETE_TRIGGER_RETENTION_CONSTRAINT:
				$triggerMsg = 'retention constraing';
				break;
			case \OCA\Files_Versions\Storage::DELETE_TRIGGER_QUOTA_EXCEEDED:
				$triggerMsg = 'versions quota exceeded';
				break;
			default:
				$triggerMsg = 'unknown';
		}

		$logger->log(
			'Version delete "{path}", trigger: {trigger}',
			[
				'path' => self::getAbsolutePath($params[Filesystem::signal_param_path]),
				'trigger' => $triggerMsg
			]
		);
	}

	static public function version_rollback($params) {
		$logger = self::getLogger();

		$logger->log(
			'Version of "{path}" restored to revision "{revision}"',
			[
				'path' => self::getAbsolutePath($params[Filesystem::signal_param_path]),
				'revision' => $params['revision']
			]
		);
	}
}
