<?php
/**
 * ownCloud Admin_Audit
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright (C) 2016 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit;

use OCP\ILogger;
use OCP\IRequest;
use OCP\IUser;
use OCP\IUserSession;
use OCP\SystemTag\ISystemTag;
use OCP\SystemTag\ISystemTagManager;
use OCP\SystemTag\ISystemTagObjectMapper;
use OCP\SystemTag\TagNotFoundException;

class Logger {
	/** @var ILogger */
	protected $logger;

	/** @var IRequest */
	protected $request;

	/** @var IUserSession */
	protected $session;

	/** @var ISystemTagManager */
	protected $tagManager;

	/** @var ISystemTagObjectMapper */
	protected $tagMapper;

	/**
	 * Logger constructor.
	 *
	 * @param ILogger $logger
	 * @param IRequest $request
	 * @param IUserSession $session
	 * @param ISystemTagManager $tagManager
	 * @param ISystemTagObjectMapper $tagMapper
	 */
	public function __construct(ILogger $logger, IRequest $request, IUserSession $session, ISystemTagManager $tagManager, ISystemTagObjectMapper $tagMapper) {
		$this->logger = $logger;
		$this->request = $request;
		$this->session = $session;
		$this->tagManager = $tagManager;
		$this->tagMapper = $tagMapper;
	}

	/**
	 * @return null|IUser
	 */
	public function getSessionUser() {
		return $this->session->getUser();
	}

	/**
	 * @return string
	 */
	public function getUserOrIp() {
		$user = $this->getSessionUser();
		if ($user instanceof IUser) {
			return sprintf('user <%s>, displayName <%s>', $user->getUID(), $user->getDisplayName());
		} else if ($this->request->getRemoteAddress() !== '') {
			return 'IP ' . $this->request->getRemoteAddress();
		} else {
			return 'cron';
		}
	}

	/**
	 * @param string $fileId
	 * @param string[] $includeTags
	 * @param string[] $excludeTags
	 * @return string
	 */
	public function getTagStringForFile($fileId, array $includeTags = [], array $excludeTags = []) {
		$mapping = $this->tagMapper->getTagIdsForObjects($fileId, 'files');

		if (empty($mapping[$fileId]) && empty($includeTags)) {
			// No tags
			return '';
		}

		$tagIds = $mapping[$fileId];
		if (!empty($excludeTags)) {
			$tagIds = array_diff($tagIds, $excludeTags);
		}
		if (!empty($includeTags)) {
			$tagIds = array_merge($tagIds, $includeTags);
		}

		if (empty($tagIds)) {
			// No tags left
			return '';
		}

		try {
			$tags = $this->tagManager->getTagsByIds($tagIds);
		} catch (TagNotFoundException $e) {
			try {
			// Ignore missing tags
			$tags = $this->tagManager->getTagsByIds(array_diff($tagIds, $e->getMissingTags()));
			} catch (TagNotFoundException $e) {
				// Again an error, ignore and go back...
				return '';
			}
		}

		return ' ["' . implode('", "', array_map([$this, 'prepareTagAsParameter'], $tags)) . '"]';
	}

	/**
	 * @param ISystemTag $tag
	 * @return string
	 */
	public function prepareTagAsParameter(ISystemTag $tag) {
		if (!$tag->isUserVisible()) {
			return $tag->getName() . ' (invisible)';
		} else if (!$tag->isUserAssignable()) {
			return $tag->getName() . ' (restricted)';
		} else {
			return $tag->getName();
		}
	}

	/**
	 * @param string $message
	 * @param string[] $parameters
	 */
	public function log($message, array $parameters = []) {
		if (\OC::$CLI) {
			$message .= ' [CLI]';
		} else {
			$message .= ' [CLIENT_IP: {ip}]';
			$message .= ' [USER_AGENT: {ua}]';
		}

		$userAgent = $this->request->getHeader('USER_AGENT');
		if(is_null($userAgent)) {
			$userAgent = 'unknown';
		}

		$orignalUser = '';
		if (\OCP\App::isEnabled('impersonate')) {
			$orignalUser = \OC::$server->getSession()->get('oldUserId');
			if ($orignalUser !== null) {
				$orignalUser = "user $orignalUser ( impersonated as " . $this->getUserOrIp() . " )";
			}
		}

		if ($orignalUser === '' or $orignalUser === null) {
			$orignalUser = $this->getUserOrIp();
		}

		$this->logger->info($message, array_merge([
			'ip'	=> $this->request->getRemoteAddress(),
			'ua'	=> $userAgent,
			'actor'	=> $orignalUser,
			'app'	=> 'admin_audit',
		], $parameters));
	}
}
